# coding=utf-8

import re

from lxml import etree

from . import exceptions


class Node(object):

    """Node object."""

    def __new__(cls, name, value=None, parent=None, **kwargs):
        """Create object.

        :param name: name of node.
        :param value (optional): text of node.
        :param parent (optional): `etree.Element` instance parent of node.
        """
        element = etree.Element(name, **kwargs)

        if value:
            try:
                element.text = str(value)
            except UnicodeEncodeError:
                element.text = u'{}'.format(value)

        if parent is not None:
            parent.append(element)

        return element


class XMLObject(object):

    """XML object."""

    @classmethod
    def from_node(cls, node):
        """Load values from node.

        :param node: `etree.Element` instance.
        """
        if not etree.iselement(node):
            raise ValueError('Is not an `etree.Element` instance.')

        if node.tag != cls.__name__:
            raise exceptions.MalformedEntityError(
                '{} not a "{}" tag.'.format(node.tag, cls.__name__)
            )

    def to_node(self):
        """Serialize to `etree.Element` instance.

        :returns: `etree.Element` instance.
        """
        raise NotImplementedError('Object should contains method `to_node`.')

    def to_xml(self, pretty_print=False):
        """Serialize to XML string.

        :param pretty_print (optional): `bool` human-readable XML.
        :returns: `str` XML.
        """
        return etree.tostring(self.to_node(), pretty_print=pretty_print)


class Label(XMLObject):

    """Label object."""

    def __init__(self, value, name='Details',
                 content_type='text/html'):
        """Initialization object.

        :param value: label value.
        :param name (optional): label name.
        :param content_type (optional): label content type.
        """
        self.name = name
        self.value = value
        self.content_type = content_type

    @classmethod
    def from_node(cls, node):
        """Load values from node.

        :param node: `etree.Element` instance.
        :returns: `entities.Label` instance.
        """
        super(Label, cls).from_node(node)

        return cls(
            node.attrib['Name'], node.text.strip(), node.attrib.get('Type', '')
        )

    def to_node(self):
        """Serialize to `etree.Element` instance.

        :returns: `etree.Element` instance.
        """
        node = Node(self.__class__.__name__)

        node.attrib['Name'] = self.name
        node.attrib['Type'] = self.content_type
        node.text = etree.CDATA(self.value)

        return node


class Field(XMLObject):

    """Field object."""

    def __init__(self, name, value, display_name=None, matching_rule=None):
        """Initialization object.

        :param name: field name.
        :param value: field value.
        :param display_name (optional): field display name.
        :param matching_rule (optional): field matching rule.
        """
        self.name = name
        self.value = value
        self.display_name = display_name or ' '.join(
            re.split(r'[\._\s-]', self.name)
        ).title()
        self.matching_rule = matching_rule

    @classmethod
    def from_node(cls, node):
        """Load values from node.

        :param node: `etree.Element` instance.
        :returns: `entities.Field` instance.
        """
        super(Field, cls).from_node(node)

        return cls(
            node.attrib['Name'], node.text.strip(),
            node.attrib.get('DisplayName'), node.attrib.get('MatchingRule')
        )

    def to_node(self):
        """Serialize to `etree.Element` instance.

        :returns: `etree.Element` instance.
        """
        node = Node(self.__class__.__name__, self.value)

        node.attrib['Name'] = self.name
        node.attrib['DisplayName'] = self.display_name

        if self.matching_rule is not None:
            node.attrib['MatchingRule'] = self.matching_rule

        return node


class Entity(XMLObject):

    """Entity base object."""

    def __init__(self, name, value, weight=None, icon_url=None,
                 fields=None, labels=None):
        """Initialization object.

        :param name : `str` entity name.
        :param value : `str` entity value.
        :param weight (optional): `str` entity weight.
        :param icon_url (optional): `str` entity icon url.
        :param fields (optional): `list` fields.
        :param labels (optional): `list` labels.
        """
        self.name = name
        self.value = value
        self.weight = weight
        self.icon_url = icon_url
        self.fields = fields or []
        self.labels = labels or []

    @classmethod
    def from_node(cls, node):
        """Load values from node.

        :param node: `etree.Element` instance.
        :returns: `entities.Entity` instance.
        """
        super(Entity, cls).from_node(node)

        try:
            name = node.attrib['Type']
        except KeyError:
            raise exceptions.MalformedEntityError('No "Type" attribute.')

        value = node.find('Value')
        if value is None:
            raise exceptions.MalformedEntityError('Missing "Value" tag.')

        value = value.text and value.text.strip()

        instance = cls(name, value)

        weight = node.find('Weight')
        if weight is not None:
            instance.weight = weight.text and weight.text.strip()

        additional_fields = node.find('AdditionalFields')
        if additional_fields is not None:
            for field_node in additional_fields.getchildren():
                field = Field.from_node(field_node)
                instance.fields.append(field)

        labels = node.find('DisplayInformation')
        if labels is not None:
            for label_node in labels.getchildren():
                label = Label.from_node(label_node)
                instance.labels.append(label)

        icon_url = node.find('IconURL')
        if icon_url is not None:
            instance.icon_url = icon_url.text and icon_url.text.strip()

        return instance

    def to_node(self):
        """Serialize to `etree.Element` instance.

        :returns: `etree.Element` instance.
        """
        node = Node(self.__class__.__name__)
        node.attrib['Type'] = self.name

        Node('Value', self.value, parent=node)

        if self.weight is not None:
            Node('Weight', self.weight, parent=node)

        if self.fields:
            additional_fields = Node('AdditionalFields', parent=node)
            for field in self.fields:
                if field.value:
                    additional_fields.append(field.to_node())

        if self.labels:
            labels = Node('DisplayInformation', parent=node)
            for label in self.labels:
                labels.append(label.to_node())

        if self.icon_url:
            Node('IconURL', value=self.icon_url, parent=node)

        return node



# coding=utf-8

from lxml import etree

from . import entities, exceptions, constants


class MaltegoMessage(entities.XMLObject):

    """Maltego message object."""

    def __init__(self):
        """Override initialization."""
        self.entities = []
        self.fields = {}
        self.soft_limit = constants.DEFAULT_SOFT_LIMIT
        self.hard_limit = constants.DEFAULT_HARD_LIMIT
        self.ui_messages = []

    @classmethod
    def from_node(cls, node):
        """Load values from node.

        :param node: `etree.Element` instance.
        :returns: `etree.Element` node.
        """
        if not etree.iselement(node):
            raise ValueError('Is not an `etree.Element` instance.')

        node = node.getchildren()[0]

        if node.tag != 'Maltego{}Message'.format(cls.__name__):
            raise exceptions.MalformedMessageError(
                '{} is invalid MaltegoMessage Type.'.format(node.tag)
            )

        return node

    def to_node(self):
        """Serialize to `etree.Element` instance.

        :returns: `etree.Element` instance.
        """
        node = entities.Node(
            'Maltego{}Message'.format(self.__class__.__name__)
        )

        if self.ui_messages:
            ui_messages = entities.Node('UIMessages', parent=node)
            for message in self.ui_messages:
                entities.Node(
                    'UIMessage', message['value'], parent=ui_messages,
                    attrib={'MessageType': message['type']}
                )
        return node

    def to_xml(self, pretty_print=False):
        """Serialize to XML string.

        :param pretty_print (optional): `bool` human-readable XML.
        :returns: `str` XML.
        """
        message = entities.Node('MaltegoMessage')
        message.append(self.to_node())
        return etree.tostring(message, pretty_print=pretty_print)


class TransformRequest(MaltegoMessage):

    """Maltego transform request message object."""

    @classmethod
    def from_xml(cls, xml):
        """Create object from xml.

        :param xml: `str` XML.
        :returns: `messages.TransformRequest` instance.
        """
        return cls.from_node(etree.fromstring(xml))

    @classmethod
    def from_node(cls, node):
        """Load values from node.

        :param node: `etree.Element` instance.
        :returns: `messages.TransformRequest` instance.
        """
        node = super(TransformRequest, cls).from_node(node)

        entity_nodes = node.find('Entities')
        if entity_nodes is None:
            raise exceptions.MalformedMessageError(
                'Request requires "Entities" tag.'
            )

        instance = cls()

        for entity in entity_nodes.getchildren():
            instance.entities.append(entities.Entity.from_node(entity))

        fields = node.find('TransformFields')
        if fields is not None:
            for field in fields.getchildren():
                if 'Name' not in field.attrib:
                    raise exceptions.MalformedMessageError(
                        'No "Name" attribute in Field'
                    )
                name = field.attrib['Name']
                value = field.text and field.text.strip()
                instance.fields[name] = value

        limit = node.find('Limits')
        if limit is not None:
            instance.soft_limit = int(
                limit.attrib.get('SoftLimit', constants.DEFAULT_SOFT_LIMIT)
            )
            instance.hard_limit = int(
                limit.attrib.get('HardLimit', constants.DEFAULT_SOFT_LIMIT)
            )

        return instance


class TransformResponse(MaltegoMessage):

    """Maltego transform response message."""

    def __init__(self, entities_iter, ui_messages=None):
        """Override initialization.

        :param entities: iterable object of `entities.Entity` instances.
        """
        super(TransformResponse, self).__init__()
        self.entities = list(entities_iter)
        self.ui_messages = ui_messages or []

    def to_node(self):
        """Serialize to `etree.Element` instance.

        :returns: `etree.Element` instance.
        """
        node = super(TransformResponse, self).to_node()

        entities_node = entities.Node('Entities', parent=node)
        for entity in self.entities:
            entities_node.append(entity.to_node())

        return node



# coding=utf-8

from . import messages


class BaseTransform(object):

    """Base transform object."""

    def __init__(self, message):
        """Initialization class.

        :param message: `messages.MaltegoMessage` subclasses instance.
        """
        if not issubclass(message.__class__, messages.MaltegoMessage):
            raise ValueError(
                'message should be instance of'
                ' `messages.MaltegoMessage` subclasses.'
            )
        self.message = message

    def transform(self):
        """Do transform.

        :returns: iterable object of `entities.Entity` instances.
        """
        raise NotImplementedError('Object should contains method `transform`.')

    def to_response(self):
        """Create `messages.TransformResponse` instance.

        :returns: `messages.TransformResponse` instance.
        """
        return messages.TransformResponse(self.transform())



#!/usr/bin/env python
# coding=utf-8

from os import path
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

VERSION = '1.1.6'

BASE_DIR = path.abspath(path.dirname(__file__))

PACKAGES = [
    'pymaltego'
]

REQUIRES = [
    'lxml'
]

with open(path.join(BASE_DIR, 'README.rst')) as f:
    LONG_DESCRIPTION = f.read()

setup(
    name='pymaltego',
    version=VERSION,
    packages=PACKAGES,
    install_requires=REQUIRES,
    description='Package for developing Maltego Transforms',
    long_description=LONG_DESCRIPTION,
    author='Vitalii Maslov',
    author_email='me@pyvim.com',
    url='https://github.com/pyvim/pymaltego',
    download_url='https://github.com/pyvim/pyvim/tarball/master',
    license='MIT',
    keywords='Maltego, transforms',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
    ],
)
